/*
*  Name:      application.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC application
*  Date:      $Date: 1999/04/20 14:04:07 $
*  Revision:  $Revision: 1.8 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_APPLICATION__
#define _UESQLC_APPLICATION__

#include <cstdio>
#include <string>
#include <mpcl/extended_map.hh>
#include <mpcl/smart_pointer.hh>
#include <mpcl/pattern_based_code_generator.hh>
#include <mpcl/config_processor.hh>
#include <mpcl/base_application.hh>
#include "defs.hh"
#include "requirement_verifier.hh"


/*
*  U S E D   N A M E S P A C E S
*/

using namespace mpcl;


class TApplication : public TBaseApplication
{

  protected:

    typedef 
      TSmartPointer <TPatternBasedCodeGenerator>  
      QTPatternBasedCodeGenerator;

    typedef
      xextended_map<string, XVariableMap>
      XTargetToRuntimeParametersMapMap;


  protected:

    /*
    *  A T T R I B U T E S
    *
    *    tConfigProcessor
    *      Configuration processor.
    *
    *    tRequirementVerifier
    *      Requirement verifier for compiler.
    *
    *    uiCfgDebug
    *      Debug level from configuration stage.
    *
    *    gCfgHelp
    *      Boolean that is true if <help> option has been parsed in
    *      configuration stage.
    *
    *    gCfgVersion
    *      Boolean that is true if <version> option has been parsed
    *      in configuration stage.
    *
    *    gCfgParameters
    *      Boolean that is true if <parameters printing> option has
    *      been parsed in configuration stage.
    *
    *    gCfgOutputCdml
    *      Boolean that is true if <output cdml> option has been
    *      parsed in configuration stage.
    *
    *    gCfgOutputCdmlSkel
    *      Boolean that is true if <output cdml skel> option has
    *      been parsed in configuration stage.
    *
    *    gCfgLineDirectives
    *      Boolean that is true if <no line directives> option has
    *      been parsed in configuration stage.
    *
    *    yCfgInputFileName
    *      String with the name of the input file name.
    *
    *    yCfgOutputFileName
    *      String with the name of the output file name.
    *
    *    yCfgCdmlPath
    *      String with the path for CDML files.
    *
    *    yCfgTarget
    *      String with target platform.
    *
    *    gCompilationOk
    *      Boolean with compilation successfulness.
    *
    *    gReusingTarget
    *      True if the current target has been loaded before.
    *
    *    tReservedWordMap
    *      Map with correspondences between reserved words character
    *      representation and, their token value.
    *
    *    tTargetToCodeGeneratorMap
    *      Code-generators map.
    *
    *    tTargetToRuntimeParametersMapMap
    *      Map with target to runtime parameters map.
    *
    *    tCodeParametersMap
    *      Variable map with arguments for code-generator.
    *
    *    ptRuntimeCodeParametersMap
    *      Pointer to runtime variable map for code-generator.
    *
    *    yOutputCode
    *      Output code.
    *
    *    yPrefixOutputCode
    *      Prefix output code.
    *
    *    yPostfixOutputCode
    *      Postfix output code.
    *
    *    yCurrentTarget
    *      Current target name.
    *
    *    yLastToken
    *      String with the last token read from scanner in symbolic
    *      form.
    */

    TConfigProcessor                                     tConfigProcessor;
    TRequirementVerifier                                 tRequirementVerifier;
    unsigned int                                         uiCfgDebug;
    bool                                                 gCfgHelp;
    bool                                                 gCfgVersion;
    bool                                                 gCfgParameters;
    bool                                                 gCfgOutputCdml;
    bool                                                 gCfgOutputCdmlSkel;
    bool                                                 gCfgLineDirectives;
    string                                               yCfgInputFileName;
    string                                               yCfgOutputFileName;
    string                                               yCfgCdmlPath;
    string                                               yCfgTarget;
    bool                                                 gCompilationOk;
    bool                                                 gReusingTarget;
    size_t                                               zEmbeddedIntroducerLine;
    xextended_map<string, long int>                      tReservedWordMap;
    xextended_map<string, QTPatternBasedCodeGenerator>   tTargetToCodeGeneratorMap;
    XTargetToRuntimeParametersMapMap                     tTargetToRuntimeParametersMapMap;
    XVariableMap                                         tCodeParametersMap;
    XVariableMap*                                        ptRuntimeParametersMap;
    string                                               yOutputCode;
    string                                               yPrefixOutputCode;
    string                                               yPostfixOutputCode;
    string                                               yCurrentTarget;
    string                                               yLastToken;


  public:

    /*
    *  C O N S T R U C T O R S
    */

    TApplication (const char* pkcAPPLICATION_NAME, const char* pkcAPPLICATION_RELEASE);

    ~TApplication (void);
    
    void initialize (int iPARAMETERS_COUNT, const char** ppkcPARAMETERS_LIST);

    void reinitialize (void) {}
    
    int start (void)
      throw (TExceptionUESQLC_Error);
    /*
    *  Description:  Opens files, and runs the compiler.
    */

    void setCodeGenerator (const char* pkcTARGET);
    /*
    *  Description:  Loads the code-generator with the proper
    *                CDML target file.
    */

    void initCodeParameters (void);
    /*
    *  Description:  Initializes tCodeParametersMap with one values.
    *                This is "_target", that contains the current
    *                target value.
    */

    void bindCodeParameter (const string& rkyVARIABLE, const string& rkyVALUE);
    /*
    *  Description:  Bind variable rktVARIABLE with value rkyVALUE.
    *
    *  PRE  ::=  !tCodeParametersMap.isBinded (rkyVARIABLE)
    *  POST ::=  ( tCodeParametersMap.isBinded (rkyVARIABLE)        && 
    *              ( tCodeParametersMap [rkyVARIABLE] == rkyVALUE ) )
    */

    void outputCode (const char* pkcTEXT, size_t zLENGTH = 0);
    /*
    *  Description:  Append on output zLENGTH characters from
    *                pkcTEXT.
    */

    void preAppendClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME = "body");
    /*
    *  Description:  Writes at the begining of output, the code
    *                corresponding to instruction pkcINSTRUCTION_NAME
    *                and clause pkcCLAUSE_NAME.
    */

    void postAppendClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME = "body");
    /*
    *  Description:  Writes at the end of output, the code corresponding
    *                to instruction pkcINSTRUCTION_NAME and clause pkcCLAUSE_NAME.
    */

    void outputClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME = "body");
    /*
    *  Description:  Writes on output, the code corresponding to instruction
    *                pkcINSTRUCTION_NAME and clause pkcCLAUSE_NAME.
    */

    void pushTokenName (const char* pkcLAST_TOKEN);
    /*
    *  Description:  Sets the last read token from scanner.
    */

    void pushTokenName (const string& kyLAST_TOKEN);
    /*
    *  Description:  Sets the last read token from scanner.
    */

    int identifyToken (const char* pkcTEXT);
    /*
    *  Description:  Search reserved-word map for the token 
    *                binded to pkcTEXT, and if it is found,
    *                then the token is returned, else token
    *                IDENTIFIER_BODY is returned.
    */    

    void handleTarget (const char* pkcTEXT);
    /*
    *  Description:  Extracts target from EMBEDDED_SQL_BEGIN and sets
    *                the proper cdml target.
    */    

    string collapseStringLiteral (const char* pkcTEXT);
    /*
    *  Description:  Collapses multiple string_literal parts into
    *                an only one.
    */    
    

  public:

    /*
    *  S E L E C T O R S
    */

    bool isReusingTarget (void) const;
    /*
    *  Description:  Returns true if current target has been used before
    */

    bool useLineDirective (void) const;
    /*
    *  Description:  Returns true if compiler must use '#line' directives.
    */
    
    const string programFileName (void) const;
    
    string getLineDirective (bool gEMBEDDED_INTRODUCER_LINE_NUMBER = false) const;
    /*
    *  Description:  Gets line directive (if any).
    */

    string getClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME = "body");
    /*
    *  Description:  Gets clause body for clause pkcCLAUSE_NAME in
    *                instruction pkcINSTRUCTION_NAME.
    */

    void require (long int liREQUERIMENT_IDENTIFIER...) const;
    /*
    *  Description:  Verifies that requirement corresponding to
    *                liREQUERIMENT_IDENTIFIER, is true. If not,
    *                then throws the corresponding exception.
    *
    *  Declaration:  void require (long int, TStackValue*...);
    *
    *  PRE ::=  ( verify (liREQUERIMENT_IDENTIFIER) )
    */

    void handleCompilerError (const char* pkcMESSAGE) const
      throw (TExceptionUESQLC_Error);
    /*
    *  Description:  Handles compiler error (throws exception).
    */
    
    void createComplementaryFiles (void) const
      throw (TExceptionUESQLC_Error);
    /*
    *  Description:  Create complementary files (if any).
    *                These files correspond with all variables named
    *                as "file::*".
    */

    string debugDescription (void) const;
    /*
    *  Description:  Gets debug description, according with
    *                debug level.
    */    

    void printDebugInformation (void) const;
    /*
    *  Description:  Writes to standard error a debug description,
    *                according with debug level.
    */


  protected:

    /*
    *  S E L E C T O R S
    */

    void printExceptionDescription (const TBaseException& rktSOURCE_EXCEPTION) const;
    /*
    *  Description:  Writes to standard error the exception description.
    */

    void printUsage (void) const;
    /*
    *  Description:  Writes to standard output the usage information.
    */

    void printParameters (void) const;
    /*
    *  Description:  Writes to standard output the
    *                configuration parameters.
    */


  protected:

    /*
    *  C O N S T R U C T O R S
    */

    void openFiles (void);
    /*
    *  Description:  Opens files.
    */

    void closeFiles (void);
    /*
    *  Description:  Closes files.
    */

};  /* class TApplication */


/*
*  C O N S T R U C T O R S
*/

inline void TApplication::bindCodeParameter (const string& rkyVARIABLE, const string& rkyVALUE)
{

  tCodeParametersMap.bind (rkyVARIABLE, TVariableString (rkyVALUE));

}  /* bindCodeParameter() */


/*
*  E X T E R N A L   V A R I A B L E S
*/

extern char      yytext[];
extern int       yylineno;
extern int       yydebug;
extern FILE*     yyin;
extern FILE*     yyout;


/*
*  E X T E R N A L   F U N C T I O N S
*/

extern int yyparse (void);

#endif  /* not _UESQLC_APPLICATION__ */
